package de.moritzruth.dracula_musical.act

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.dracula_musical.song.*
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.PropPosition
import de.moritzruth.theaterdsl.show.ShowBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun ShowBuilderContext.act1() = act("1. Akt") {
    scene("Ouvertüre [4]") {
        step(StepCue.MusicStart("Ouvertüre", 1.minutes)) {
            curtainState = CurtainState.CLOSED

            props {
                it[PropPosition.BACKDROP] = "Dorfplatz"
            }

            onRun {
                Washs.both {
                    it.pointAtAudience()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
                    it.beamAngle.static(100.percent)
                }
            }
        }

        step(StepCue.MusicEnd) {
            actors {
                +"Van Helsing / in der Mitte der Vorbühne"
            }

            rightSpotTarget = "Van Helsing (Mitte der Vorbühne)"

            onRun {
                Spots.right.brightness.fade(100.percent, 2.seconds)
            }
        }

        step(StepCue.Text("Van Helsing", "Der Mittsommernacht.")) {
            actors {
                -"Van Helsing"
            }

            rightSpotTarget = null

            onRun {
                Spots.right.brightness.off(1.seconds)
            }
        }
    }

    scene("Mittsommernacht [4]") {
        songMittsommernacht()
    }

    scene("Ankunft in Huntington [6]") {
        lightStep(StepCue.Text("Mina", "Das ist also Huntington!")) {
            sidelight.brightness.fade(75.percent, 15.seconds)
            sidelight.color1.fadeRandomAround(130.degrees, 5.degrees, 10.seconds)
            sidelight.color2.fadeRandomAround(60.degrees, 5.degrees, 9.seconds)
            sidelight.color3.fadeRandomAround(130.degrees, 5.degrees, 12.seconds)
            sidelight.color4.fadeRandomAround(60.degrees, 5.degrees, 11.seconds)
        }
    }

    scene("Hawkins’ Feierabend [8]") {
        step(StepCue.Text("Lucy", "Heute gehört Mister Harker uns.", "Ende")) {
            actors {
                -"Mina"
                -"Lucy"
                -"Jonathan"
                +"Hawkins / von links"
                // TODO: Expand
                +"Sekretärinnen / von links"
            }

            props {
                it[PropPosition.CENTER] = "Hawkins’ Stuhl"
            }
        }

        step(StepCue.Text("Sekretärin 1", "Ashton und Ashton 6 Pfund glatt…")) {
            actors {
                +"Renfield / von links"
            }
        }

        songAdelIstGeil()

        step(StepCue.Text("Renfield", "Ich bin schon weg.")) {
            actors {
                -"Renfield / durch Mitte"
            }
        }

        step(StepCue.MusicStart("Umbau (Adel ist geil)", 1.minutes + 30.seconds)) {
            actors {
                -"Hawkins / durch Mitte"
                -"Sekretärinnen / durch Mitte"
            }

            curtainState = CurtainState.CLOSED

            props {
                it[PropPosition.BACKDROP] = "Irrenanstalt"
                it[PropPosition.CENTER] = "Medikamenten-Schrank"
            }

            onRun {
                FrontLights.center { it.brightness.off(6.seconds) }
                sidelight.brightness.off(6.seconds)
            }
        }
    }

    scene("Die Irrenanstalt [12]") {
        step(StepCue.MusicEnd) {
            actors {
                +"Ascot / durch rechte Tür"
                +"Mina / durch rechte Tür"
            }

            curtainState = CurtainState.OPEN

            onRun {
                FrontLights.center { it.brightness.fade(100.percent, 6.seconds) }
                sidelight.brightness.fade(100.percent, 6.seconds)
                sidelight.colors.forEach { it.static(Color.WARM_WHITE) }
            }
        }

        step(StepCue.Text("Ascot", "Da kommt der Rest der Familie!")) {
            actors {
                +"Oberschwester / durch rechte Tür"
                // TODO: Expand
                +"Pflegeteam / durch rechte Tür"
            }
        }

        songIrrenhaus()

        step(StepCue.Text("Oberschwester", "Sonst kriegst du sie als Zäpfchen!")) {
            actors {
                -"Oberschwester"
                -"Pflegeteam"
                -"Die Irren"
            }
        }

        step(StepCue.MusicStart("Pflegemarsch", 25.seconds)) {}

        step(StepCue.MusicEnd) {
            actors {
                +"Renfield / durch rechte Tür"
                +"Mrs. Nothgroschen / durch rechte Tür"
                +"Wärter 1 / durch rechte Tür"
                +"Wärter 2 / durch rechte Tür"
            }
        }

        step(StepCue.Text("Dr. Sewart", "Ich liebe meinen Job!")) {
            actors {
                -"Renfield / durch rechte Tür"
                -"Mrs. Nothgroschen / durch rechte Tür"
                -"Wärter 1 / durch rechte Tür"
                -"Wärter 2 / durch rechte Tür"
                -"Dr. Sewart / durch rechte Tür"
                -"Ascot / durch rechte Tür"
            }

            onRun {
                FrontLights.all { it.brightness.fade(75.percent, 10.seconds) }
            }
        }

        step(StepCue.Custom("Mina öffnet die Tür")) {
            actors {
                +"Mr. Nickelby / durch rechte Tür"
                +"Mrs. Nickelby / durch rechte Tür"
                +"Tochter Nickelby / durch rechte Tür"
            }
        }

        step(StepCue.Text("Mina", "Es gibt hier noch mehr Irre.")) {
            actors {
                -"Mina / durch rechte Tür"
                +"Lucy / von links"
            }
        }
    }

    scene("Blockflötenkreis [20]") {
        songLucy()

        step(StepCue.Custom("Gardine wird geöffnet")) {
            rightSpotTarget = null

            onRun {
                FrontLights.center { it.brightness.fade(100.percent, 2.seconds) }
                sidelight.brightness.off(3.seconds)
                Spots.right.brightness.off(3.seconds)
            }
        }

        step(StepCue.Text("Mr. Nickelby", "…als du die Vorhänge aufgemacht hast.")) {
            actors {
                +"Ascot / durch rechte Tür"
            }

            onRun {
                sidelight.colors.forEach { it.fadeRandomAround(0.degrees, 5.degrees, 3.seconds) }
            }
        }

        step(StepCue.Text("Ascot", "Sie Flötenguru!")) {
            actors {
                +"Dr. Sewart / durch rechte Tür"
            }
        }

        repeat(3) {
            lightStep(StepCue.Custom("Gardine wird geschlossen")) {
                FrontLights.center { it.brightness.fade(40.percent, 4.seconds) }
                sidelight.brightness.fade(80.percent, 3.seconds)
            }

            lightStep(StepCue.Custom("Gardine wird geöffnet")) {
                FrontLights.center { it.brightness.fade(100.percent, 2.seconds) }
                sidelight.brightness.off(2.seconds)
            }
        }

        step(StepCue.Text("Dr. Sewart", "Und dass mir ja die Gardine aufbleibt!")) {
            actors {
                -"Dr. Sewart"
                -"Ascot"
                -"Lucy"
                -"Mr. Nickelby"
                -"Mrs. Nickelby"
                -"Tochter Nickelby"
            }

            curtainState = CurtainState.CLOSED

            onRun {
                FrontLights.center { it.brightness.off(5.seconds) }
            }
        }

        step(StepCue.MusicStart("Umbau (Sittlichkeitsverein)", 10.seconds)) {}
    }

    scene("Männerbesuch [22]") {
        step(StepCue.MusicEnd) {
            actors {
                +"Mina / durch rechte Tür"
                +"Jonathan / durch rechte Tür"
            }

            curtainState = CurtainState.OPEN

            onRun {
                FrontLights.center { it.brightness.fade(75.percent, 5.seconds) }
            }
        }

        step(StepCue.Text("Mina", "Sehr merkwürdig.")) {
            actors {
                +"Ascot / durch rechte Tür"
            }
        }

        step(StepCue.Text("Ascot", "Miss Murray!", "beim dritten Mal")) {
            actors {
                -"Ascot / über rechte Treppe"
            }
        }

        songDuettMinaJonathan()

        step(StepCue.Text("Mina", "So markiere ich, was mir gehört.")) {
            actors {
                +"Ascot"
            }
        }

        step(StepCue.Text("Jonathan", "Gute Nacht.")) {
            actors {
                -"Mina / über linke Treppe"
                -"Jonathan / durch rechte Tür"
                -"Ascot"
            }

            leftSpotTarget = null
            curtainState = CurtainState.CLOSED

            onRun {
                FrontLights.center { it.brightness.off(5.seconds) }
            }
        }

        step(StepCue.MusicStart("Umbau (Duett)", 1.minutes + 35.seconds)) {}
    }

    scene("Kaffeeklatsch [24]") {
        step(StepCue.MusicEnd) {
            actors {
                +"Dr. Sewart / durch rechte Tür"
            }

            curtainState = CurtainState.OPEN
            rightSpotTarget = "Dr. Sewart (von rechts)"

            onRun {
                sidelight.colors.forEach { it.static(Color.WARM_WHITE) }
                sidelight.brightness.fade(10.percent, 6.seconds)
                Spots.right.brightness.fade(100.percent, 6.seconds)
            }
        }

        step(StepCue.Custom("Dr. Sewart ist versteckt")) {
            actors {
                +"Oberschwester / über rechte Treppe"
            }

            onRun {
                FrontLights.center { it.brightness.fade(50.percent, 4.seconds) }
                sidelight.brightness.fade(75.percent, 5.seconds)
            }
        }

        step(StepCue.Text("Dr. Sewart", "Da sind sie!")) {
            actors {
                +"Ascot / durch rechte Tür, mit Stuhl"
                // TODO: Expand
                +"Kaffeetanten / durch rechte Tür, mit Stühlen"
                // TODO: Expand
                +"Nichten / durch rechte Tür, mit Stühlen"
            }

            rightSpotTarget = null

            onRun {
                FrontLights.center { it.brightness.fade(100.percent, 3.seconds) }
                sidelight.brightness.off(10.seconds)
                Spots.right.brightness.off(6.seconds)
            }
        }

        step(StepCue.Text("Dr. Sewart", "Ich mache Diät!")) {
            actors {
                -"Dr. Sewart / durch rechte Tür"
                -"Oberschwester / nach links"
            }
        }

        songKaffeeklatsch()
    }

    scene("Ankunft Van Helsings [28]") {
        step(StepCue.Text("Ascot", "Jetzt ist Lucy wieder ganz gesund.")) {
            actors {
                +"Dr. Sewart / von rechts"
                +"Lucy / getragen von Dr. Sewart"
            }

            onRun {
                Washs.both {
                    it.pointAtCeiling()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
                    it.magenta.reset()
                }
            }
        }

        step(StepCue.Text("Dr. Sewart", "Was mach ich nur!")) {
            actors {
                +"Van Helsing / durch Mitte"
            }

            rightSpotTarget = "Van Helsing"

            onRun {
                Spots.right.brightness.pulseOnce(0.seconds, 20.seconds)
                FrontLights.center {
                    it.brightness.pulseOnce(
                        rampUpDuration = 1.seconds,
                        rampDownDuration = 10.seconds,
                        peak = 0.percent,
                        start = 100.percent,
                        end = 100.percent
                    )
                }

                BlinderBars.all {
                    it.preset.static(null)
                    it.color.static(Color.WHITE)
                    it.white.static(40.percent)
                    it.brightness.pulseOnce(0.3.seconds, 2.seconds, 100.percent)
                }

                Washs.both {
                    it.pointAtAudience()
                    it.beamAngle.fade(0.percent, 2.seconds)
                    it.brightness.fade(start = 100.percent, end = 0.percent, duration = 2.seconds)
                }
            }
        }

        step(StepCue.Text("Van Helsing", "Oder will jemand von den Damen?")) {
            actors {
                -"Kaffeetanten / nach rechts, 1.Tür, mit Stühlen"
                -"Nichten / nach rechts, 1.Tür, mit Stühlen"
            }
        }

        step(StepCue.Text("Van Helsing", "Helfen Sie mir jetzt!")) {
            actors {
                -"Dr. Sewart / nach links"
                -"Lucy / gezogen von Dr. Sewart"
                -"Van Helsing / nach links"
                -"Ascot / durch rechte Tür, mit Stuhl"
            }

            curtainState = CurtainState.CLOSED
            rightSpotTarget = null

            props {
                it[PropPosition.CENTER] = null
                it[PropPosition.LEFT] = "3 Särge"
                it[PropPosition.BACKDROP] = "Draculas Schloss"
            }

            onRun {
                FrontLights.center { it.brightness.off(3.seconds) }

                BlinderBars.all {
                    it.color.reset()
                    it.white.reset()
                }
            }
        }
    }

    scene("Verträge [30]") {
        step(StepCue.Custom("Vorhang ist geschlossen")) {
            actors {
                +"Hawkins / durchs Publikum"
                +"Jonathan / durchs Publikum"
            }

            rightSpotTarget = "Hawkins & Jonathan (im Publikum)"

            onRun {
                Spots.right.brightness.fade(100.percent, 3.seconds)
            }
        }

        step(StepCue.Text("Hawkins", "…zum Grafen Dracula!")) {
            actors {
                -"Hawkins / durch den Vorhang"
                -"Jonathan / durch den Vorhang"
            }

            rightSpotTarget = "mittig verbleiben"

            onRun {
                Spots.right.brightness.off(10.seconds)
            }
        }
    }

    scene("Einbruch [31]") {
        step(StepCue.Custom("Umbau abgeschlossen")) {
            actors {
                +"Backe / von links, mit Lampe"
                +"Timmy / von links, mit Lampe"
                +"Eloïse / im Sarg"
                +"Françoise / im Sarg"
                +"Bernadette / im Sarg"
            }

            curtainState = CurtainState.OPEN

            onRun {
                sidelight.color1.fadeRandomAround(230.degrees, 30.degrees, 2.seconds)
                sidelight.color2.fadeRandomAround(230.degrees, 35.degrees, 4.seconds)
                sidelight.color3.static(Color(hue = 230.degrees, saturation = 20.percent))
                sidelight.color4.static(Color(hue = 230.degrees, saturation = 20.percent))
                sidelight.brightness.fade(40.percent, 8.seconds)
            }
        }

        songEsIstAngerichtet()
        songDraculasZorn()
    }

    scene("Ein Sammlerstück [37]") {
        step(StepCue.Custom("Die Vampire saugen Timmy aus")) {
            actors {
                +"Jonathan / von rechts, mit Vertrag & Amulett"
            }

            onRun {
                backlightBar.brightness.fade(2.percent, 6.seconds)
            }
        }

        songFinaleErsterAkt()
    }
}