package de.moritzruth.dracula_musical.act

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.dracula_musical.song.*
import de.moritzruth.theaterdsl.device.reset
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.PropPosition
import de.moritzruth.theaterdsl.show.ShowBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.seconds

fun ShowBuilderContext.act2() = act("2. Akt") {
    scene("Maskenball [43]") {
        songMaskenball()

        step(StepCue.Text("Van Helsing", "Na dann, gute Nacht.")) {
            actors {
                -"Van Helsing / durch Mitte"
                -"Dr. Sewart / durch Mitte"
                +"Dracula / durch Mitte"
                +"Jonathan / durch Mitte"
            }
        }

        step(StepCue.Text("Hawkins", "Geil!")) {
            actors {
                -"Hawkins / nach links"
            }
        }

        step(StepCue.Text("Ascot", "Bringen Sie Mister Harker zur Tür!")) {
            actors {
                -"Jonathan / durch Mitte"
                -"Ascot / durch Mitte"
                -"Oberschwester / durch Mitte"
                -"Lucy / durch Mitte"
            }
        }
    }

    scene("Tanz mit einem Vampir [47]") {
        songDuettDraculaMina()

        step(StepCue.Custom("Mina präsentiert Dracula ihren Hals")) {
            actors {
                +"Ascot / durch Mitte"
                +"Oberschwester / durch Mitte"
                +"Lucy / durch Mitte"
            }

            rightSpotTarget = null

            onRun {
                Washs.both { it.brightness.off(0.4.seconds) }
                backlightBar.brightness.off(0.4.seconds)
                sidelight.brightness.off(0.4.seconds)
                FrontLights.center { it.brightness.fade(100.percent, 3.seconds) }
            }
        }

        step(StepCue.Text("Ascot", "Oberschwester!")) {}

        step(StepCue.Text("Ascot", "Oberschwester!")) {
            actors {
                -"Ascot / durch Mitte"
                -"Oberschwester / durch Mitte"
                -"Mina / durch Mitte"
            }

            onRun {
                FrontLights.center { it.brightness.fade(40.percent, 20.seconds) }
            }
        }

        step(StepCue.Text("Dracula", "Darf ich bitten?")) {
            actors {
                -"Dracula / durch Mitte"
                -"Lucy / durch Mitte"
            }
        }

        songRepriseMaskenball()
    }

    scene("Van Helsing [51]") {
        step(StepCue.Text("Tante 1", "Verhungert!")) {
            actors {
                +"Hawkins / durch Mitte"
                +"Dr. Sewart / durch Mitte"
                +"Van Helsing / durch Mitte"
            }
        }

        step(StepCue.Text("Van Helsing", "Alle raus, habe ich gesagt!")) {
            actors {
                -"Besucher des Maskenballs / durch Mitte"
                -"Kaffeetanten / durch Mitte"
                -"Hawkins / durch Mitte"
            }

            onRun {
                FrontLights.center { it.brightness.fade(75.percent, 10.seconds) }
                backlightBar.brightness.fade(25.percent, 10.seconds)

                Tops.both {
                    it.startRoomMovement(2.0)
                    it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.White)
                }
            }
        }

        lightStep(StepCue.Custom("Van Helsing zückt das Kreuz (mit Musik)")) {
            BlinderBars.all {
                it.brightness.fade(100.percent, 2.seconds)
                it.preset.static(StairvilleSplb.Preset.STUB)  // switching, hectic
                it.presetSpeed.static(100.percent)
            }

            Tops.both {
                it.pointAtStageMiddleSingleCenterMarking()
                it.strobeSpeed.rampDown(4.seconds)
                it.brightness.static(100.percent)
            }

            FrontLights.center { it.brightness.fade(100.percent, 2.seconds) }
        }

        step(StepCue.Custom("Lucy ist geflüchtet")) {
            actors {
                -"Lucy"
            }

            onRun {
                Tops.both {
                    it.brightness.off()
                    it.strobeSpeed.off()
                }
                BlinderBars.all { it.brightness.off(1.seconds) }
                FrontLights.center { it.brightness.fade(75.percent, 1.seconds) }
            }
        }

        songVanHelsing()

        step(StepCue.Text("Van Helsing", "… stirbt sie einen qualvollen Tod.")) {
            actors {
                +"Hawkins"
            }
        }

        step(StepCue.Text("Van Helsing", "Dann schnell!")) {
            actors {
                -"Van Helsing"
                -"Dr. Sewart"
            }
        }

        step(StepCue.Text("Hawkins", "… ein Bad nehmen – vorher!")) {
            actors {
                -"Hawkins"
            }
        }

        step(StepCue.MusicStart("Umbau (Adel ist geil)", 10.seconds)) {
            curtainState = CurtainState.CLOSED

            props {
                it[PropPosition.BACKDROP] = "Irrenanstalt"
                it[PropPosition.CENTER] = "Medikamenten-Schrank"
            }

            onRun {
                FrontLights.center { it.brightness.off(10.seconds) }
                backlightBar.brightness.off(15.seconds)
            }
        }
    }

    scene("Wiederholungstäter [54]") {
        step(StepCue.MusicEnd) {
            actors {
                +"Mina / durch rechte Tür"
                +"Ascot / durch rechte Tür"
            }

            curtainState = CurtainState.OPEN

            onRun {
                FrontLights.center { it.brightness.fade(40.percent, 8.seconds) }

                sidelight.color1.fadeRandomAround(230.degrees, 30.degrees, 2.seconds)
                sidelight.color2.fadeRandomAround(230.degrees, 35.degrees, 4.seconds)
                sidelight.color3.static(Color(hue = 230.degrees, saturation = 20.percent))
                sidelight.color4.static(Color(hue = 230.degrees, saturation = 20.percent))
                sidelight.brightness.fade(50.percent, 8.seconds)
            }
        }

        step(StepCue.Text("Ascot", "… an der Nase herumführen!")) {
            actors {
                -"Mina / nach links"
                -"Ascot / nach links"
                +"Backe / durch Mitte"
                +"Timmy / durch Mitte"
            }

            onRun {
                FrontLights.center { it.brightness.off(8.seconds) }
            }
        }

        step(StepCue.Text("Backe", "Nein, dir, du Pfeife!")) {
            actors {
                +"Ascot / von links"
            }

            onRun {
                FrontLights.center { it.brightness.fade(40.percent, 5.seconds) }
            }
        }

        step(StepCue.Text("Backe", "… und ruckzuck sind alle glücklich.")) {
            actors {
                -"Ascot / durch rechte Tür"
                -"Backe / durch rechte Tür"
                -"Timmy / durch rechte Tür"
                +"Dracula / von rechts"
                +"Renfield / hinter Fenster"
            }

            rightSpotTarget = "Dracula (rechts)"
        }

        lightStep(StepCue.Custom("Dracula erscheint")) {
            Spots.right.brightness.fade(100.percent, 2.seconds)
        }

        step(StepCue.Text("Dracula", "Still wie eine Leiche. Idiot.")) {
            actors {
                -"Renfield / tot"
                -"Dracula / durch rechte Tür"
                +"Jonathan / durch rechte Tür"
            }

            rightSpotTarget = null

            onRun {
                Spots.right.brightness.off(2.seconds)
            }
        }
    }

    scene("So viele Vampire! [57]") {
        step(StepCue.Text("Jonathan", "Ich bin’s! Jonathan!")) {
            actors {
                +"Eloïse / durch Mitte"
                +"Françoise / durch Mitte"
                +"Bernadette / durch Mitte"
                +"Lucy / von links"
            }
        }

        songQuartett()

        step(StepCue.Custom("Die Vampire nähern sich Jonathan")) {
            actors {
                +"Van Helsing / durch Mitte"
                +"Dr. Sewart / von rechts"
            }

            onRun {
                Tops.both {
                    it.startRoomMovement(2.0)
                    it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.White)
                }
            }
        }

        lightStep(StepCue.Custom("Van Helsing zückt das Kreuz (mit Musik)")) {
            Tops.both {
                it.pointAtStageMiddleSingleCenterMarking()
                it.strobeSpeed.rampDown(4.seconds)
                it.brightness.static(100.percent)
            }

            FrontLights.center { it.brightness.fade(100.percent, 2.seconds) }

            backlightBar.brightness.pulseOnce(1.seconds, 3.seconds)

            BlinderBars.all {
                it.brightness.fade(100.percent, 2.seconds)
                it.preset.static(StairvilleSplb.Preset.STUB)  // switching, hectic
                it.presetSpeed.static(100.percent)
            }
        }

        step(StepCue.Custom("Die Vampire sind geflüchtet")) {
            actors {
                -"Bernadette / nach links"
                -"Eloïse / durch Mitte"
                -"Françoise / nach rechts"
                -"Lucy / nach rechts"
            }

            onRun {
                Tops.both { it.reset() }
                BlinderBars.all { it.brightness.off(1.seconds) }
                FrontLights.center { it.brightness.fade(75.percent, 1.seconds) }
            }
        }

        step(StepCue.Custom("Van Helsing öffnet die Tür")) {
            actors {
                +"Dracula / hinter 2. Tür von rechts"
                +"Mina / in Draculas Armen"
            }

            rightSpotTarget = "Dracula & Mina (2. Tür von rechts)"

            onRun {
                Spots.right.brightness.static(100.percent)
                Tops.both { it.startRoomMovement(2.0) }
                Washs.both {
                    it.pointAtCeiling()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
                }
            }
        }

        lightStep(StepCue.Custom("Van Helsing zückt das Kreuz (mit Musik)")) {
            BlinderBars.all {
                it.brightness.fade(100.percent, 2.seconds)
                it.preset.static(StairvilleSplb.Preset.STUB)  // switching, hectic
                it.presetSpeed.static(100.percent)
            }

            Tops.both {
                it.pointAtStageMiddleSingleCenterMarking()
                it.strobeSpeed.rampDown(4.seconds)
                it.brightness.static(100.percent)
            }

            FrontLights.center { it.brightness.fade(100.percent, 2.seconds) }
            Spots.right.brightness.off(5.seconds)

            Washs.both {
                it.pointAtAudience(2.seconds)
                it.brightness.rampUp(1.seconds)
            }
        }

        step(StepCue.Custom("Dracula ist geflüchtet")) {
            actors {
                -"Dracula"
                -"Mina"
            }

            rightSpotTarget = null

            onRun {
                BlinderBars.all { it.brightness.off(1.seconds) }
                Tops.both { it.reset() }
                Washs.both { it.brightness.off() }
                FrontLights.center { it.brightness.fade(75.percent, 1.seconds) }
            }
        }

        lightStep(StepCue.Custom("Musik: Einsetzen der Drums")) {
            BlinderBars.all {
                it.preset.static(StairvilleSplb.Preset.STUB)  // chase, back-and-forth, hectic
                it.presetSpeed.static(40.percent)
                it.brightness.fade(40.percent, 0.5.seconds)
            }
        }

        step(StepCue.Text("Van Helsing", "Wir müssen sie finden! Schnell!")) {
            actors {
                -"Van Helsing / durch rechte Tür"
                -"Dr. Sewart / durch rechte Tür"
                -"Jonathan / durch rechte Tür"
            }

            curtainState = CurtainState.CLOSED  // fast

            onRun {
                FrontLights.center { it.brightness.off(3.seconds) }
                sidelight.brightness.off(3.seconds)
                BlinderBars.all { it.brightness.off(2.seconds) }
            }
        }

        step(StepCue.MusicStart("Umbau (Van Helsing)", 10.seconds)) {}
    }

    scene("Uneinigkeit [61]") {
        step(StepCue.MusicEnd) {
            actors {
                +"Backe / von rechts"
                +"Timmy / von rechts"
            }

            curtainState = CurtainState.OPEN

            onRun {
                sidelight.color1.fadeRandomAround(230.degrees, 30.degrees, 2.seconds)
                sidelight.color2.fadeRandomAround(230.degrees, 35.degrees, 4.seconds)
                sidelight.color3.static(Color(hue = 230.degrees, saturation = 20.percent))
                sidelight.color4.static(Color(hue = 230.degrees, saturation = 20.percent))
                sidelight.brightness.fade(50.percent, 8.seconds)
                FrontLights.center { it.brightness.fade(75.percent, 8.seconds) }
            }
        }

        step(StepCue.Text("Backe", "Bis auf deine neue Freundin.")) {
            actors {
                +"Ascot / von rechts"
            }
        }

        step(StepCue.Text("Ascot", "Und wie sieht’s denn hier aus!")) {
            actors {
                -"Ascot / nach links"
            }
        }

        step(StepCue.Text("Backe", "Gib mal einen Schluck Blut.")) {
            actors {
                +"Eloïse / von rechts"
                +"Françoise / von rechts"
                +"Bernadette / von rechts"
                +"Lucy / von rechts"
            }
        }

        step(StepCue.Text("Eloïse", "Immer direkt von der Quelle trinken.")) {
            actors {
                +"Ascot / von links"
            }
        }

        songStreitDerVampire()

        step(StepCue.Text("Vampir 3", "Wenn sie es nicht ist — wer dann?")) {
            actors {
                +"Hawkins / durch Mitte"
            }
        }

        step(StepCue.Text("Hawkins", "Geil!")) {
            actors {
                +"Jonathan / von links"
                +"Van Helsing / von links"
                +"Dr. Sewart / von links"
            }
        }

        step(StepCue.Text("Jonathan", "Wo ist Mina?")) {
            actors {
                +"Dracula / von links"
                +"Mina / von links"
            }
        }

        lightStep(StepCue.Custom("Van Helsing zückt das Kreuz (mit Musik)")) {
            BlinderBars.all {
                it.brightness.fade(50.percent, 2.seconds)
                it.preset.static(StairvilleSplb.Preset.STUB)  // switching, hectic
                it.presetSpeed.static(100.percent)
            }
        }

        lightStep(StepCue.Custom("Hawkins entreißt Van Helsing das Kreuz")) {
            BlinderBars.all {
                it.brightness.off(2.seconds)
                it.presetSpeed.off(0.5.seconds)
            }

            Washs.both {
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red)
                it.beamAngle.static(0.percent)
                it.pointAtCeiling()
            }
        }

        lightStep(StepCue.Custom("Musik: Setzt ein")) {
            backlightBar.color.static(Color.RED)
            backlightBar.brightness.pulse(1.8.seconds, 0.15.seconds, 0.2.seconds, peak = 40.percent, startDelay = 0.seconds)

            Washs.both { it.brightness.pulse(1.8.seconds, 0.15.seconds, 0.3.seconds, startDelay = 0.4.seconds) }
        }

        lightStep(StepCue.Custom("Der Vorhang wird geöffnet")) {
            FrontLights.center { it.brightness.static(100.percent) }

            backlightBar.color.fade(Color.WARM_WHITE, 0.5.seconds)
            backlightBar.brightness.pulseOnce(0.5.seconds, 25.seconds, end = 40.percent)

            sidelight.colors.forEach { it.fade(Color.WARM_WHITE, 0.5.seconds) }
            sidelight.brightness.pulseOnce(0.5.seconds, 25.seconds, end = 40.percent)

            Washs.both {
                it.pointAtStageCenter()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
                it.brightness.pulseOnce(0.5.seconds, 25.seconds)
            }

            BlinderBars.all {
                it.preset.static(null)
                it.white.static(100.percent)
                it.color.reset()
                it.brightness.pulseOnce(0.5.seconds, 10.seconds)
            }
        }
    }

    scene("Liebe macht blind [67]") {
        step(StepCue.Text("Van Helsing", "Natürlich. Kommen Sie.")) {
            actors {
                -"Van Helsing / durch Mitte"
                -"Dr. Sewart / durch Mitte"
                -"Dracula / ist verdampft"
                -"Eloïse / ist verdampft"
                -"Françoise / ist verdampft"
                -"Bernadette / ist verdampft"
                -"Ascot / ist verdampft"
                -"Backe / ist verdampft"
                -"Timmy / ist verdampft"
                -"Lucy / ist verdampft"
                -"Hawkins / ist verdampft"
            }

            leftSpotTarget = "Jonathan"

            onRun {
                FrontLights.center { it.brightness.off(20.seconds) }
                Spots.left.brightness.fade(100.percent, 10.seconds)
                backlightBar.brightness.off(20.seconds)
                sidelight.brightness.fade(20.percent, 20.seconds)
            }
        }

        songRepriseDuett()

        lightStep(StepCue.Custom("Jonathan zieht Minas Sarg")) {
            Spots.left.brightness.off(6.seconds)
            backlightBar.color.static(Color.DARK_BLUE)
            backlightBar.brightness.fade(40.percent, 10.seconds)

            Tops.both {
                it.startRoomMovement(2.0)
                it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.DarkRed)
                it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
            }
        }

        lightStep(StepCue.Custom("Musik: Wechsel in moll")) {

            sidelight.colors.forEach { it.fade(Color.RED, 10.seconds) }
            sidelight.brightness.fade(40.percent, 10.seconds)
        }

        lightStep(StepCue.Custom("Musik: Einsetzen der Orgel")) {
            backlightBar.color.fade(Color(hue = 359.degrees), 6.seconds)
            backlightBar.brightness.fade(60.percent, 5.seconds)

            Washs.both { it.brightness.pulse(1.05.seconds, 0.1.seconds, 0.2.seconds) }
        }

        lightStep(StepCue.Custom("Musik: +2 Takte")) {
            sidelight.brightness.fade(75.percent, 5.seconds)
            backlightBar.brightness.pulse(1.05.seconds, 0.1.seconds, 0.2.seconds)

            Washs.both { it.brightness.pulse(1.05.seconds, 0.1.seconds, 0.2.seconds, startDelay = 0.3.seconds) }
        }

        lightStep(StepCue.Custom("1. Beat")) {
            BlinderBars.all {
                it.preset.static(null)
                it.white.static(100.percent)
                it.color.reset()
                it.brightness.pulseOnce(0.seconds, 0.2.seconds)
            }

            sidelight.brightness.off(0.1.seconds)
            backlightBar.brightness.off(0.1.seconds)
        }

        step(StepCue.Custom("2. Beat")) {
            rightSpotTarget = "Jonathan’s Kopf (sofort von weit auf schmal ziehen)"

            onRun {
                Spots.right.brightness.fade(start = 100.percent, end = 0.percent, duration = 2.seconds)
            }
        }

        lightStep(StepCue.Custom("3. Beat")) {
            BlinderBars.all {
                it.white.reset()
                it.color.static(Color.RED)
                it.brightness.pulseOnce(0.seconds, 0.2.seconds)
            }

            sidelight.brightness.pulseOnce(0.seconds, 1.seconds)

            Tops.both { it.brightness.pulseOnce(0.seconds, 0.4.seconds) }
        }

        lightStep(StepCue.Custom("4. Beat")) {
            backlightBar.brightness.pulseOnce(0.seconds, 10.seconds)
        }
    }

    scene("Finale [68]") {
        songFinale()
    }
}