package de.moritzruth.dracula_musical.device

import de.moritzruth.theaterdsl.device.DeviceGroup
import de.moritzruth.theaterdsl.dmx.DmxAddress
import de.moritzruth.theaterdsl.dmx.DmxValue
import kotlinx.collections.immutable.persistentSetOf

object FrontLights {
    private const val GAMMA = 2.0
    private val MINIMUM_VISIBLE_VALUE = DmxValue(60u)
    val left = DeviceGroup(SimpleDimmer(DmxAddress(1u), GAMMA, MINIMUM_VISIBLE_VALUE), SimpleDimmer(DmxAddress(2u), GAMMA, MINIMUM_VISIBLE_VALUE))
    val center = DeviceGroup(SimpleDimmer(DmxAddress(4u), GAMMA, MINIMUM_VISIBLE_VALUE), SimpleDimmer(DmxAddress(5u), GAMMA, MINIMUM_VISIBLE_VALUE))
    val right = DeviceGroup(SimpleDimmer(DmxAddress(7u), GAMMA, MINIMUM_VISIBLE_VALUE), SimpleDimmer(DmxAddress(8u), GAMMA, MINIMUM_VISIBLE_VALUE))
    val all = left + center + right
}

object Spots {
    private const val GAMMA = 2.0
    private val MINIMUM_VISIBLE_VALUE = DmxValue(40u)
    val left = SimpleDimmer(DmxAddress(10u), GAMMA, MINIMUM_VISIBLE_VALUE)
    val right = SimpleDimmer(DmxAddress(11u), GAMMA, MINIMUM_VISIBLE_VALUE)
}

object Tops {
    val left = FuturelightDmh160(DmxAddress(37u), false)
    val right = FuturelightDmh160(DmxAddress(53u), true)
    val both = DeviceGroup(left, right)
}

object Washs {
    val left = CoemarProWash(DmxAddress(101u), false)
    val right = CoemarProWash(DmxAddress(85u), true)
    val both = DeviceGroup(left, right)
}

object BlinderBars {
    val inner = DeviceGroup(StairvilleSplb(DmxAddress(155u)), StairvilleSplb(DmxAddress(178u)))
    val outer = DeviceGroup(StairvilleSplb(DmxAddress(201u)), StairvilleSplb(DmxAddress(224u)))
    val all = inner + outer
}

val backlightBar = StairvilleTlb(DmxAddress(130u))
val sidelight = StairvilleClb4(DmxAddress(140u))

val fogMachine = AdjFogFury3000(DmxAddress(117u))

val devices = persistentSetOf(
    *FrontLights.all.toTypedArray(),
    Spots.left,
    Spots.right,
    *Tops.both.toTypedArray(),
    *Washs.both.toTypedArray(),
    *BlinderBars.all.toTypedArray(),
    backlightBar,
    sidelight,
    fogMachine
)