package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.device.reset
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.show.greedyCueStep
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songLucy() {
    lightStep(StepCue.MusicStart("Lucy", 2.minutes + 25.seconds)) {
        sidelight.brightness.fade(30.percent, 15.seconds)
        backlightBar.brightness.fade(40.percent, 15.seconds)
        backlightBar.color.static(Color(hue = 315.degrees, saturation = 20.percent))

        FrontLights.left { it.brightness.off(10.seconds) }
        FrontLights.right { it.brightness.off(10.seconds) }

        Washs.both {
            it.pointAtCeiling()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
            it.magenta.static(100.percent)
            it.beamAngle.static(100.percent)
        }

        Tops.both {
            it.startRoomMovement(4.0)
            it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.LightPink)
            it.prismMode.static(FuturelightDmh160.PrismMode.OPEN)
        }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        backlightBar.brightness.pulseOnce(0.4.seconds, 4.seconds, end = 40.percent)

        BlinderBars.all {
            it.brightness.pulseOnce(0.4.seconds, 2.5.seconds, peak = 75.percent)
            it.preset.static(null)
            it.white.static(100.percent)
        }
    }

    lightStep(StepCue.Text("Lucy", "Sittlichkeitsverein", "letzte Silbe")) {
        backlightBar.color.fade(Color(hue = 315.degrees, saturation = 100.percent), 30.seconds)
        backlightBar.brightness.sine(3.seconds, minimum = 40.percent, maximum = 75.percent)

        BlinderBars.all {
            it.brightness.fade(25.percent, 2.seconds)
            it.preset.static(StairvilleSplb.Preset.STUB)  // warm/red/pink, theatre-like, flowy
            it.presetSpeed.static(40.percent)
        }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, end = 25.percent) }

        Washs.left.brightness.sine(6.seconds, start = 0.percent)
        Washs.right.brightness.sine(6.seconds, start = 100.percent)
    }

    lightStep(StepCue.Text("Lucy", "Du hast den besten Hintern…", "Anfang")) {
        Tops.both { it.brightness.sine(8.seconds, 20.percent, 100.percent) }
    }

    greedyCueStep(StepCue.Text("Lucy", "Oh Nick…"))

    lightStep(StepCue.Text("Lucy", "oh Nick", "letzte Silbe")) {
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 1.seconds) }
    }

    step(StepCue.Custom("Musik: Interludium")) {
        rightSpotTarget = "Lucy"

        onRun {
            FrontLights.center { it.brightness.fade(50.percent, 2.seconds) }
            Spots.right.brightness.fade(100.percent, 1.seconds)
            sidelight.brightness.off(2.seconds)
        }
    }

    lightStep(StepCue.Custom("Musik: Langes Fill-in beginnt")) {
        BlinderBars.all {
            it.presetSpeed.fade(100.percent, 5.seconds)
            it.brightness.fade(50.percent, 5.seconds)
        }
    }

    lightStep(StepCue.Custom("Musik: Fill-in endet")) {
        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 1.5.seconds, end = 25.percent)
            it.presetSpeed.static(40.percent)
        }

        FrontLights.center { it.brightness.fade(100.percent, 1.seconds) }

        Tops.both { it.brightness.sine(5.seconds, 20.percent, 100.percent) }
    }

    lightStep(StepCue.Text("Lucy", "oh Nick", "letzte Silbe")) {
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 1.seconds) }

        Washs.both {
            it.brightness.off(0.1.seconds)
            it.pointAtStageCenter()
        }

        BlinderBars.all {
            it.brightness.fade(10.percent, 0.4.seconds)
            it.presetSpeed.static(60.percent)
        }
    }

    lightStep(StepCue.Text("Lucy", "Mein Sexidol", "erste Silbe")) {
        BlinderBars.all { it.brightness.fade(20.percent, 0.2.seconds) }
    }

    lightStep(StepCue.Text("Lucy", "Mein Sexidol", "zweite Silbe")) {
        BlinderBars.all { it.brightness.fade(35.percent, 0.2.seconds) }
    }

    lightStep(StepCue.Text("Lucy", "Mein Sexidol", "dritte Silbe")) {
        BlinderBars.all { it.brightness.fade(50.percent, 0.2.seconds) }
    }

    lightStep(StepCue.Text("Lucy", "Mein Sexidol", "vierte Silbe")) {
        BlinderBars.all {
            it.brightness.pulseOnce(1.seconds, 2.5.seconds, end = 25.percent)
            it.presetSpeed.fade(25.percent, 1.seconds)
        }
    }

    lightStep(StepCue.Custom("1. Beat")) {
        BlinderBars.all { it.brightness.off(0.5.seconds) }
        FrontLights.center { it.brightness.off(0.5.seconds) }
    }

    greedyCueStep(StepCue.Custom("2. Beat"))

    lightStep(StepCue.Custom("3. Beat")) {
        backlightBar.brightness.pulseOnce(0.1.seconds, 2.seconds)

        BlinderBars.all {
            it.preset.static(null)
            it.white.static(100.percent)
            it.brightness.pulseOnce(0.1.seconds, 2.seconds, peak = 60.percent)
        }
    }

    lightStep(StepCue.Custom("Musik: Schlussschlag")) {
        backlightBar.color.static(Color.RED)
        backlightBar.brightness.pulseOnce(0.1.seconds, 5.seconds)

        Washs.both { it.brightness.pulseOnce(0.1.seconds, 3.seconds) }
        FrontLights.center { it.brightness.pulseOnce(0.1.seconds, 4.seconds, end = 40.percent) }
    }

    step(StepCue.MusicEnd) {
        Tops.both { it.reset() }
        Washs.both { it.reset() }
        BlinderBars.all { it.reset() }
    }
}