package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.PropPosition
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songMaskenball() {
    step(StepCue.MusicStart("Maskenball", 3.minutes + 10.seconds)) {
        props {
            it[PropPosition.BACKDROP] = "Dorfplatz"
        }
    }

    step(StepCue.Custom("Musik: Vierter Glockenschlag")) {
        curtainState = CurtainState.OPEN

        onRun {
            backlightBar.color.static(Color.DARK_BLUE)
            backlightBar.brightness.fade(50.percent, 5.seconds)
        }
    }

    step(StepCue.Custom("Musik: Einsetzen der Drums")) {
        actors {
            +"Hawkins / von links"
            +"Dr. Sewart / von links"
            +"Eloïse / von links"
            +"Françoise / von links"
            +"Bernadette / von links"
            // TODO: Expand
            +"Besucher des Maskenballs / von links & durch Mitte"
        }

        onRun {
            backlightBar.brightness.off(4.seconds)

            FrontLights.center { it.brightness.fade(100.percent, 2.seconds) }

            Tops.both {
                it.startRoomMovement(5.0)
                it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Rotate(20.percent))
                it.prismMode.static(FuturelightDmh160.PrismMode.FACETS_3)
                it.prismRotationSpeed.static(10.percent)
            }

            Washs.both {
                it.pointAtCeiling()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(80.percent))
            }
        }
    }

    lightStep(StepCue.Text("Alle", "Maskenball!", "Anfang")) {
        backlightBar.color.cycle(8.seconds)
        backlightBar.brightness.fade(80.percent, 5.seconds)

        Tops.both { it.brightness.fade(100.percent, 1.seconds) }
        Washs.both { it.brightness.fade(100.percent, 1.seconds) }
        FrontLights.center { it.brightness.fade(100.percent, 2.seconds) }

        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // theatre-like, warm
            it.presetSpeed.static(40.percent)
            it.brightness.fade(25.percent, 2.seconds)
        }
    }

    lightStep(StepCue.Custom("Musik: Breakdown am Ende")) {
        BlinderBars.all { it.presetSpeed.fade(0.percent, 5.seconds) }
        Washs.both { it.brightness.off(5.seconds) }
    }

    lightStep(StepCue.Custom("Musik: Schlussschlag")) {
        backlightBar.color.cycle(20.seconds)
        Tops.both { it.brightness.off(5.seconds) }
        Washs.both { it.brightness.pulseOnce(0.2.seconds, 5.seconds) }
        BlinderBars.all { it.brightness.off(5.seconds) }
    }

    step(StepCue.MusicEnd) {
        actors {
            -"Eloïse / durch Mitte"
            -"Françoise / durch Mitte"
            -"Bernadette / durch Mitte"
            -"Besucher des Maskenballs / durch Mitte"
            +"Van Helsing / durch Mitte"
            +"Dr. Sewart / durch Mitte"
            +"Ascot / durch Mitte"
            +"Oberschwester / durch Mitte"
            +"Lucy / durch Mitte"
            +"Mina / steht rechts"
        }

        onRun {
            backlightBar.brightness.fade(20.percent, 5.seconds)
        }
    }
}