package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songVanHelsing() {
    step(StepCue.MusicStart("Van Helsing", 2.minutes + 30.seconds)) {
        rightSpotTarget = "Van Helsing"

        onRun {
            Spots.right.brightness.fade(100.percent, 20.seconds)
        }
    }

    lightStep(StepCue.Text("Van Helsing", "Dann werde ich es Ihnen sagen.")) {
        backlightBar.brightness.off(10.seconds)
        FrontLights.left { it.brightness.off(10.seconds) }
        FrontLights.center { it.brightness.fade(25.percent, 10.seconds) }
        FrontLights.right { it.brightness.off(10.seconds) }
    }

    lightStep(StepCue.Text("Van Helsing", "Menschenblut…", "Anfang")) {
        backlightBar.brightness.sine(0.75.seconds, minimum = 50.percent, maximum = 100.percent)
        backlightBar.color.static(Color(hue = 200.degrees, saturation = 40.percent))

        Tops.both {
            it.startRoomMovement(5.0)
            it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.DarkBlue)
            it.prismMode.static(FuturelightDmh160.PrismMode.OPEN)
        }

        Washs.both {
            it.pointAtCeiling()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.DarkBlue)
            it.beamAngle.static(100.percent)
        }
    }

    lightStep(StepCue.Custom("Musik: Beat nach »Kreatur des Bösen«")) {
        BlinderBars.all {
            it.preset.static(null)
            it.color.reset()
            it.white.static(100.percent)
            it.brightness.pulseOnce(0.1.seconds, 0.3.seconds)
        }

        Tops.both { it.brightness.pulseOnce(0.1.seconds, 0.3.seconds) }
    }

    lightStep(StepCue.Text("Van Helsing", "Ein höllischer Dämon ist hier!", "letzte Silbe")) {
        BlinderBars.all { it.brightness.pulseOnce(2.seconds, 0.3.seconds, peak = 50.percent) }
        Washs.both {
            it.pointAtAudience(3.seconds)
            it.brightness.pulseOnce(2.seconds, 0.3.seconds, peak = 60.percent)
        }
    }

    lightStep(StepCue.Text("Van Helsing", "Ein Vampir!", "Anfang")) {
        Washs.both { it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red) }
    }

    lightStep(StepCue.Text("Van Helsing", "Ein Vampir!", "letzte Silbe")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 1.seconds, peak = 50.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 1.seconds, peak = 100.percent) }
    }

    lightStep(StepCue.Custom("Musik: Beat nach »Meute«")) {
        BlinderBars.all {
            it.preset.static(null)
            it.color.reset()
            it.white.static(100.percent)
            it.brightness.pulseOnce(0.1.seconds, 0.3.seconds)
        }

        Tops.both { it.brightness.pulseOnce(0.1.seconds, 0.3.seconds) }

        Washs.both {
            it.pointAtCeiling()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.DarkBlue)
        }
    }

    lightStep(StepCue.Text("Van Helsing", "Ein höllischer Dämon ist hier!", "letzte Silbe")) {
        BlinderBars.all { it.brightness.pulseOnce(2.seconds, 0.3.seconds, peak = 50.percent) }
        Washs.both {
            it.pointAtAudience(3.seconds)
            it.brightness.pulseOnce(2.seconds, 0.3.seconds, peak = 60.percent)
        }
    }

    lightStep(StepCue.Text("Van Helsing", "Ein Vampir!", "Anfang")) {
        Washs.both { it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red) }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        backlightBar.brightness.off(0.1.seconds)
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 50.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 100.percent) }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        backlightBar.brightness.pulseOnce(0.1.seconds, 0.6.seconds)
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 50.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 100.percent) }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        backlightBar.brightness.pulseOnce(0.1.seconds, 0.6.seconds)
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 0.3.seconds) }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 50.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 100.percent) }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        backlightBar.brightness.pulseOnce(0.1.seconds, 0.6.seconds)
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 0.3.seconds) }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 50.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.6.seconds, peak = 100.percent) }
        Tops.both { it.pointAtStageMiddleSingleCenterMarking() }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        backlightBar.brightness.fade(100.percent, 0.1.seconds)
        Tops.both { it.brightness.fade(100.percent, 0.1.seconds) }
        Spots.right.brightness.off(2.seconds)
    }

    step(StepCue.MusicEnd) {
        rightSpotTarget = null

        onRun {
            Tops.both { it.brightness.off(4.seconds) }
            FrontLights.center { it.brightness.fade(75.percent, 10.seconds) }
            backlightBar.brightness.fade(40.percent, 10.seconds)
            backlightBar.color.fade(Color.WARM_WHITE, 10.seconds)
        }
    }
}