package de.moritzruth.theaterdsl.value

import kotlin.math.abs

data class Color(
    val hue: Angle,
    val saturation: Percentage = 100.percent,
    val brightness: Percentage = 100.percent
) {
    fun getRGB(): Triple<Percentage, Percentage, Percentage> {
        val c = brightness.value * saturation.value
        val h = hue.degrees / 60f
        val x = c * (1 - abs(h.mod(2f) - 1))
        val m = brightness.value - c

        val r: Double
        val g: Double
        val b: Double

        when {
            h < 1 -> {
                r = c
                g = x
                b = 0.0
            }

            h < 2 -> {
                r = x
                g = c
                b = 0.0
            }

            h < 3 -> {
                r = 0.0
                g = c
                b = x
            }

            h < 4 -> {
                r = 0.0
                g = x
                b = c
            }

            h < 5 -> {
                r = x
                g = 0.0
                b = c
            }

            else /* h < 6 */ -> {
                r = c
                g = 0.0
                b = x
            }
        }

        return Triple(Percentage(r + m), Percentage(g + m), Percentage(b + m))
    }

    companion object {
        val WHITE = Color(0.degrees, 0.percent, 100.percent)
        val BLACK = Color(0.degrees, 0.percent, 0.percent)
        val RED = Color(0.degrees, 100.percent, 100.percent)
        val DARK_BLUE = Color(240.degrees, 100.percent, 100.percent)

        val WARM_WHITE = Color(30.degrees, 40.percent, 100.percent)
    }
}