package de.moritzruth.theaterdsl.value

import kotlinx.serialization.Serializable

@Serializable
@JvmInline
value class Percentage(val value: Double) : Comparable<Percentage> {
    companion object {
        val VALUE_RANGE: ClosedFloatingPointRange<Double> = 0.0..1.0
    }

    init {
        require(value in VALUE_RANGE) { "value ($value) must be in 0..1" }
    }

    fun ofRange(range: ClosedFloatingPointRange<Double>): Double = value.transfer(VALUE_RANGE, range)

    override fun compareTo(other: Percentage): Int = value.compareTo(other.value)

    override fun toString(): String = "${value * 100}%"

    class Range(
        override val start: Percentage,
        override val endInclusive: Percentage
    ) : ClosedFloatingPointRange<Percentage> {
        override fun lessThanOrEquals(a: Percentage, b: Percentage): Boolean = a.value <= b.value

        fun asDoubleRange(): ClosedFloatingPointRange<Double> = start.value..endInclusive.value

        companion object {
            val FULL: Range = Range(VALUE_RANGE.start.asPercentage(), VALUE_RANGE.endInclusive.asPercentage())
        }
    }
}

inline val Int.percent: Percentage
    get() = try {
        Percentage(this / 100.0)
    } catch (e: IllegalArgumentException) {
        throw IllegalArgumentException("must be in 0..100")
    }

inline val Double.percent: Percentage
    get() = try {
        Percentage(this / 100f)
    } catch (e: IllegalArgumentException) {
        throw IllegalArgumentException("must be in 0..100")
    }

fun Double.asPercentage(): Percentage = Percentage(this)
